// version with intro panels

#ifdef KERNEL_H
#error Double include
#endif
#define KERNEL_H
#if _MSC_VER > 1000
#pragma once
#endif
#ifndef __AFXWIN_H__
#error include 'stdafx.h' before including this file for PCH
#endif

#define DataDir   "Data\\"
#define ErrorFile DataDir "Error.txt"

enum FileConstants {
    MAXFNAME = 256,
    MAXCNAME =  32,
    GARBLE   = 0x049e1c03
};

enum Runlevel {
    Run_Init,
    Run_Normal,
    Run_Suspend,
    Run_Title,
    Run_TitleSuspend,
    Run_Abort,
    Run_ScreenError,
    Run_MemError,
    Run_FileError,
    Run_LogError
};

enum MetaKeys {
    Key_None    = 0,
    Key_Shift   = 1,
    Key_Control = 2,
    Key_Alt     = 4,
    Key_MouseL  = 8
};

enum ViewerEvent {
    VE_None,
    VE_LeftClick,
    VE_MidClick,
    VE_RightClick,
    VE_Update,
    VE_Draw,
    VE_Display,
    VE_DisplayFade,
    VE_Drag,
    VE_Hover,
};

extern signed long  MediaRef    [];
extern char         AppName     [];
extern char         AppID       [];
extern char*        MkStr       (long);
extern int          MkNum       (char*);

#ifdef USE_GLUT
extern void __cdecl GlutKey     (unsigned char, int, int);
extern void __cdecl GlutSKey    (int, int, int);
extern void __cdecl GlutMouse   (int, int);
extern void __cdecl GlutClick   (int, int, int, int);
extern void __cdecl GlutDraw    (void);
extern void __cdecl GlutSuspend (int);
extern void __cdecl GlutIdle    (void);
#endif

#define Div(a,b) ((b) == 0 ? (a) * 2 : ((a) / (b)) + ((a) % (b) ? 1 : 0))
#define random(q) ((int)(((long)rand()*(q))/0x8000u))

#ifndef LogicalError
#ifdef _DEBUG
#define LogicalError {                      \
    TRACE2 ("*** Logical error in %s line " \
	   "%d ***\n", __FILE__, __LINE__); \
    theApp.runlevel = Run_LogError;         }
#else
#define LogicalError {                      \
    theApp.DebugMessage (__FILE__);         \
    theApp.DebugMessage (MkStr (__LINE__)); \
    theApp.runlevel = Run_LogError;         }
#endif
#endif

class CFileList {
    public:     CFileList    (void);
    void        Insert       (const char*);
    void        Remove       (CFileList*);
    void        Delete       (void);
    CFileList   *next;
    char        fname        [MAXFNAME];
};

class CIntroDlg : public CDialog {
    public:  CIntroDlg      (void);

    private:
    HICON    m_hIcon;

	//{{AFX_DATA(CIntroDlg)
	CButton RegBtn;
	//}}AFX_DATA
    //{{AFX_VIRTUAL(CIntroDlg)
	virtual BOOL PreTranslateMessage (MSG*);
    virtual void DoDataExchange      (CDataExchange*);
	//}}AFX_VIRTUAL
    //{{AFX_MSG(CIntroDlg)
    afx_msg void OnCfg      (void);
    afx_msg void OnInfo     (void);
    afx_msg void OnRegist   (void);
    afx_msg void OnWeb      (void);
	afx_msg void OnPaint();
	//}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

class CConfigDlg : public CDialog {
    public:  CConfigDlg     (void);
    void     OnCancel       (void);

	//{{AFX_DATA(CConfigDlg)
	//}}AFX_DATA
    //{{AFX_VIRTUAL(CConfigDlg)
    virtual void DoDataExchange      (CDataExchange*);
	//}}AFX_VIRTUAL
    //{{AFX_MSG(CConfigDlg)
	//}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

class CRegDlg : public CDialog {
    public:  CRegDlg        (void);
    bool     first;

	//{{AFX_DATA(CRegDlg)
	CEdit   Ed_Userkey;
	CEdit   Ed_Regkey;
	//}}AFX_DATA
    //{{AFX_VIRTUAL(CRegDlg)
	public:
	virtual BOOL PreTranslateMessage (MSG*);
    virtual void DoDataExchange      (CDataExchange*);
	//}}AFX_VIRTUAL
    //{{AFX_MSG(CRegDlg)
    afx_msg void OnForm (void);
    afx_msg void OnOK   (void);
	afx_msg void OnPaint();
	//}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

class CPieterApp : public CWinApp {
    public:
    FILE      *MediaLib;
    unsigned  runlevel,
	      regcode,
	      usercode,
	      speed,
	      speedadjust,
	      day,
	      month;
    int       detailmode,
	      network;
    char      CmpName          [MAXCNAME],
	      HomeDir          [MAXFNAME];
    CFileList flist,
	      *fcur;
    void      ProgBar          (bool first = false);
    void      DebugMessage     (char*, bool onscr = false);
    bool      CfgOpen          (bool create = false);
    void      CfgClose         (void);
    void      CfgRead          (char*, char*, long);
    void      CfgWrite         (char*, char*);
    bool      ChangeDirectory  (const char*, bool create = false);
    bool      FindFile         (const char*);
    bool      FindNext         (void);
    void      FileAttr         (const char*, unsigned);
    void      AddToRecentFiles (const char*);
    char*     CommandLine      (void);
    unsigned  DeleteFile       (const char*);
    unsigned  CopyFile         (const char*, const char*);
    WIN32_FIND_DATA filestruct;

    void      HandleInit       (void);
    void      HandleTick       (void);
    void      HandleDraw       (void);
    void      HandleExit       (void);
    void      HandleKeyPress   (unsigned, unsigned);
    void      HandleKeyUp      (unsigned, unsigned);
    void      HandleMouseMove  (unsigned, unsigned, unsigned);
    void      HandleMouseClick (unsigned, unsigned);
    void      HandleMouseUnClk (unsigned, unsigned);
    void      HandleSuspend    (bool);

    private:
    void      GetRegInfo       (char*, unsigned);
    void      InitStrTable     (void);
    void      KillClipboard    (void);
    HKEY      key;
    void      *fh;
    char      *last_msg;

    //{{AFX_VIRTUAL(CSavageaApp)
    public:
    virtual BOOL InitInstance (void);
    virtual int Run           (void);
    //}}AFX_VIRTUAL
    //{{AFX_MSG(CSavageaApp)
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

extern CPieterApp theApp;

//{{AFX_INSERT_LOCATION}}
