#ifdef PARSE_H
#error Double include.
#endif
#define PARSE_H

enum {
    MAX_SCOPE = 255,
    MAX_IDENT = 255,
    MAX_PARM  =  16
};

enum SkipReasons {
    Skip_None,
    Skip_If,
    Skip_While,
    Skip_For,
    Skip_For1,
    Skip_For2,
    Skip_Stack,
    Skip_Break,
    Skip_Func
};

enum Actions {
    P_Start,
    P_Restart,
    P_NewVar,
    P_NewVarEnd,
    P_NewArrayEnd,
    P_NewArrayEnd1,
    P_NewFunc,
    P_NewFuncEnd,
    P_FuncParm,
    P_Value,
    P_ValueEnd,
    P_Command,
    P_CmdVar,
    P_Then,
    P_SwitchEnd,
    P_Switch_0,
    P_Switch,
    P_SwBreak_0,
    P_SwBreak,
    P_ArrayAsg,
    P_ArrayRef,
    P_ArrayRef1,
    P_Case,
    P_CaseRun,
    P_Do,
    P_Loop,
    P_For0,
    P_For1,
    P_For2,
    P_For3,
    P_For4,
    P_VarRef,
    P_FuncRef,
    P_FuncRef1,
    P_FuncCall,
    P_FuncEntry,
    P_FuncEnt1,
    P_Unknown,
};

class CVar {
    public:
    unsigned value;
    char     name        [MAX_IDENT + 1];
    unsigned scope,
             tokenref,
             arraysize,
             *arraydata;
    bool     is_func;
    CVar     *next;
};

class CScope {
    public:
    unsigned action,
             retpos,
             skip,
             value,
             *varptr;
};

class CParser: public CScope {
    public:  CParser      (void);
             ~CParser     (void);
    bool     ReadScript   (unsigned, unsigned, char*);
    unsigned FunctionCall (const char*, unsigned p0=0, unsigned p1=0, unsigned p2=0);
    bool     MakeByteCode (void);
    void     Start        (void);
    void     SaveVars     (void);
    void     LoadVars     (void);
    unsigned Process      (void);
    void     PrintError   (unsigned);
    unsigned tracelevel,
             number,
             curline,
             error,
             hold;
    char     outbuf       [512];

    private:
    void     AddVar       (unsigned, unsigned);
    void     AddVar       (const char*, unsigned);
    void     KillVars     (void);
    void     PushScope    (unsigned);
    void     PopScope     (void);
    unsigned GetSpcToken  (const char*);
    unsigned SysCall      (unsigned);
    void     Output       (const char*);
    unsigned Constant     (unsigned);
    void     Trace        (unsigned, const char*, const char* e1 = " ", const char* e2 = " ");
    unsigned scrlen,
             pos,
             mainpos,
             end,
             outline,
             free_scr,
             scopelevel,
             funcparmno,
             funcparm     [MAX_PARM],
             bytecode     [0xffff],
             linenum      [0xffff];
    char     *script;
    CScope   scopestack   [MAX_SCOPE];
    CVar     *varlist,
             *funcptr;
};

extern CParser Script;
